@@ -11,15 +11,18 @@ A simple read-only XML file viewer.
 
 import sys
 import os
+import io
 import xml.etree.ElementTree as etree
 
 from plib.ui import __version__
 from plib.ui.defs import *
 from plib.ui.app import PApplication
+from plib.ui.fileaware import PFileAware
 from plib.ui.widgets import *
 
 
 def get_xml_node_tree(node):
+    # Translate node into form suitable for tree view widget
     cols = (
         node.tag,
         ", ".join("{}={}".format(key, value) for key, value in node.items()),
@@ -31,15 +34,31 @@ def get_xml_node_tree(node):
 
 
 def get_xml_tree(filename):
-    # Load XML file and return a tree of PTreeView data
+    # Parse XML file into a tree of PTreeView data
     xml = etree.parse(filename)
     return [get_xml_node_tree(xml.getroot())]
 
 
+class XMLFileLoader(PFileAware):
+    
+    def __init__(self, app):
+        PFileAware.__init__(self, app, [
+            ("XML Files", ["xml"]),
+            ("All Files", ["*"]),
+        ], file_path=app.main_path)
+    
+    def open_data(self, filename):
+        self.app.add_file(filename, get_xml_tree(filename))
+        self.close_file()  # to be ready for the next load
+    
+    def save_data(self, filename):
+        pass  # this app does not edit/save files
+
+
 class XMLViewer(PApplication):
     
     about_data = {
-        'name': "PXMLViewer",
+        'name': "XMLViewer",
         'version': "{} on Python {}".format(
             __version__,
             sys.version.split()[0]
@@ -77,22 +96,36 @@ class XMLViewer(PApplication):
     view_header = ["Tag", "Attrs", "Text", "Tail"]
     view_font_size = 16 if sys.platform == 'darwin' else 12
     
+    filenames_to_open = None
+    
+    def check_file_open(self, filename):
+        if os.path.isfile(filename):
+            print("Opening", filename)
+            self.loader.open_filename(filename)
+        else:
+            print(filename, "not found!")
+            self.filenames_not_found.append(filename)
+    
     def after_create(self):
         self.filenames = []
+        self.loader = XMLFileLoader(self)
+        self.filenames_not_found = []
+        for filename in (self.filenames_to_open or ()):
+            self.check_file_open(filename)
     
-    def add_file(self, filename):
+    def add_file(self, filename, data):
         self.filenames.append(filename)
-        data = get_xml_tree(filename)
         font_size = self.view_font_size
         view = self.view_class(self, self.tabwidget_files, labels=self.view_header, data=data, auto_expand=True,
                                font=("Arial", font_size), header_font=("Arial", font_size, True))
-        self.tabwidget_files.append((os.path.basename(filename), view))
-        self.tabwidget_files.set_current_index(len(self.tabwidget_files) - 1)
+        self.tabwidget_files.append_and_focus((os.path.basename(filename), view))
+    
+    def on_app_shown(self):
+        if self.filenames_not_found:
+            self.message_box.error("File Error", "Files {} not found".format(self.filenames_not_found))
     
     def on_file_open(self):
-        filename = self.file_dialog.open_filename(self.main_path, "*.xml")
-        if filename:
-            self.add_file(filename)
+        self.loader.open_file()
     
     def set_status_text(self, text):
         print(text)
@@ -112,4 +145,6 @@ class XMLViewer(PApplication):
 
 
 if __name__ == "__main__":
+    if len(sys.argv) > 1:
+        XMLViewer.filenames_to_open = sys.argv[1:]
     XMLViewer().run()
