@@ -127,6 +127,9 @@ class PWidgetBase(PSignalBase):
         """
         raise NotImplementedError
     
+    def get_size(self):
+        return self.get_width(), self.get_height()
+    
     def set_size(self, width, height):
         """Placeholder for derived classes to implement.
         """
@@ -339,15 +342,39 @@ class PFileDialogBase(PDialogBase):
         """
         raise NotImplementedError
     
-    def open_filename(self, path, filter):
+    def _translate_filter_item(self, caption, ext_str):
+        """Translate filter item into toolkit format.
+        """
+        raise NotImplementedError
+    
+    ext_sep = None
+    
+    def _ext_str_from_list(self, ext_list):
+        return self.ext_sep.join("*.{}".format(ext) for ext in ext_list)
+    
+    filter_sep = None
+    
+    def _translate_filter(self, filter):
+        return self.filter_sep.join(
+            self._translate_filter_item(caption, self._ext_str_from_list(ext_list))
+            for caption, ext_list in filter
+        )
+    
+    def _open_filename(self, path, filter, selected_filter):
         """Display select file dialog and return chosen filename for opening.
         """
         raise NotImplementedError
     
-    def save_filename(self, path, filter):
+    def _save_filename(self, path, filter, selected_filter):
         """Display select file dialog and return chosen filename for saving.
         """
         raise NotImplementedError
+    
+    def open_filename(self, path, filter, selected_filter):
+        return self._open_filename(path, self._translate_filter(filter), self._translate_filter_item(*selected_filter))
+    
+    def save_filename(self, path, filter, selected_filter):
+        return self._save_filename(path, self._translate_filter(filter), self._translate_filter_item(*selected_filter))
 
 
 class PAboutDialogBase(PDialogBase):
