@@ -37,6 +37,14 @@ class PTabWidgetBase(PWidgetBase, baselist):
         """
         self.extend((title, widget_from_spec(self.manager, self, *spec)) for title, spec in tabs)
     
+    def append_and_focus(self, tab):
+        """Add new tab and set focus to it.
+        
+        Assumes that tab is a (title, widget) tuple.
+        """
+        self.append(tab)
+        self.set_current_index(len(self) - 1)
+    
     def __len__(self):
         return self.tab_count()
     
