@@ -11,10 +11,11 @@ See the LICENSE and README files for more information
 import collections
 
 from plib.ui.defs import *
+from plib.ui.dialogs import DialogRunner
 from plib.ui.widgets import *
 
 
-class PPreferences(object):
+class PPreferences(DialogRunner):
     """Manage editing and saving of program preferences.
     """
     
@@ -23,11 +24,8 @@ class PPreferences(object):
     default_label_bold = False
     default_group_label_bold = True
     
-    dialog = None
-    
     def __init__(self, app, caption):
-        self.app = app
-        self.caption = caption
+        DialogRunner.__init__(self, app, caption)
         self.inifile, self.section_type, self.field_map = app.prefs_data
         self.dialog_enums = getattr(app, 'prefs_dialog_enums', {})
         self.dialog_groups = getattr(app, 'prefs_dialog_groups', {})
@@ -35,7 +33,7 @@ class PPreferences(object):
         self.group_style = getattr(app, 'prefs_group_style', self.default_group_style)
         self.label_bold = getattr(app, 'prefs_label_bold', self.default_label_bold)
         self.group_label_bold = getattr(app, 'prefs_group_label_bold', self.default_group_label_bold)
-        self.prefs_client = self.get_client_spec()
+        self.dialog_client = self.get_client_spec()
     
     def unpack_option(self, sname, opt):
         attrname = getter = setter = None
@@ -150,32 +148,26 @@ class PPreferences(object):
         method = 'set_{}'.format(self.control_method(otype, attrname))
         getattr(control, method)(value)
     
-    def load_prefs(self):
+    def populate(self):
         for otype, _, attrname in self.iter_vars():
             self.control_set(otype, attrname, getattr(self.inifile, attrname))
     
-    def show_dialog(self):
-        dialog_class = get_toolkit_class('dialog', "PDialog")
-        dialog = dialog_class(self, self.app.main_window, self.caption, self.prefs_client)
-        self.load_prefs()
-        dialog.setup_notify(SIGNAL_FINISHED, self.dialog_done)
-        dialog.display()
-    
     def control_get(self, otype, attrname):
         control = self.control_by_name(otype, attrname)
         method = 'get_{}'.format(self.control_method(otype, attrname))
         return getattr(control, method)()
     
-    def save_prefs(self):
+    def get_result(self):
         for otype, _, attrname in self.iter_vars():
             setattr(self.inifile, attrname, self.control_get(otype, attrname))
         self.inifile.writeini()
     
     def on_apply(self):
-        self.save_prefs()
+        self.get_result()
     
-    def dialog_done(self, result):
-        print("Dialog done, result:", repr(result))
-        if result:  # NOTE: we assume a simple boolean check on result is enough
-            self.save_prefs()
-        self.dialog = None
+    def dialog_done(self, accepted):
+        # We don't need to pass a callback in the constructor since we can
+        # do all the work here, we just override this method directly
+        print("Dialog accepted:", repr(accepted))
+        if accepted:
+            self.get_result()
