@@ -32,10 +32,7 @@ class PDialog(qt.QDialog, PQtWidgetBase, PDialogBase,
     def set_caption(self, caption):
         self.setWindowTitle(caption)
     
-    def size_to_controls(self, width, height):
-        self.resize(width, height)
-    
-    def display(self):
+    def do_display(self):
         self.show()
     
     def accept_changes(self):
